capture log close _all
log using rm3ch5ordinal-v3b, replace text
//  program:    rm3ch5ordinal-v3b.do
//  task:       chapter 5: ordinal
//  project:    rm3
//  versions:   v3 spost12 with non-numbered m commands
//                  b with mtable changes
//              v2 spost12 using fvs to reproduce results without fv's
//              v1 spost12 commands but not fv's
//              v0 replication against spost9
//  author:     scott 2012-09-15

//  Regression Models for Categorical Dependent Variables - 2nd Edition
//  Chapter 5 - Models for Ordinal Outcomes
//  Long and Freese - 27Jul2005

version 12.1
clear all
macro drop _all
set linesize 80
set scheme sj

local exstub "rm3ch5ordinal-v3-"
program drop _all
include spost12_which.doi

//  #1 a latent variable model

* binary model with logit
sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog
estimates store logit

* binary model with -ologit-
ologit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog
estimates store ologit

estimates table logit ologit, b(%9.3f) t label varwidth(30) equations(1:1)

//  #2 estimation of ordered regression model

sysuse ordwarm4, clear
ologit warm i.male i.white c.age c.ed c.prst if yr89==1, nolog
ologit warm i.male i.white c.age c.ed c.prst, table nolog

//  #3 attitudes toward working mothers (pg 157 RevEd)

sysuse ordwarm4
describe warm yr89 male white age ed prst
summarize warm yr89 male white age ed prst
tabulate warm

* compare -ologit- and -oprobit-
ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
estimates store ologit
oprobit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
estimates store oprobit
estimates table ologit oprobit, b(%9.3f) t label varwidth(30)

//  #4 predicting perfectly (with artifical data) (pg 160 RevEd)

* sort helps with convergence in nearly nonidentified model
sort age
gen dumprst = (prst<20 & warm==1)
tab dumprst warm, miss
ologit warm i.yr89 i.male i.white c.age c.ed i.dumprst, nolog

//  #5 testing individual coefficients (pg 161 RevEd)

//  Wald test

ologit warm i.male i.yr89 i.white c.age c.ed c.prst, nolog
test 1.male
test 1.male
display "z*z=" -9.343*-9.343
return list
di "chi2=" r(chi2) "; sqrt(chi2)= " sqrt(r(chi2))

//  LR test

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
estimates store fmodel
* estimate constrained model
ologit warm i.yr89 i.white c.age c.ed c.prst, nolog
estimates store nmodel
lrtest fmodel nmodel

//  #6 testing multiple coefficients (pg 162 RevEd)

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog

//  Wald test

*fv: test age i.white male // can't use i.
*fv: test age white male // can't use white
test age 1.white 1.male
test c.age 1.white 1.male // c. isn't needed but is ok

//  LR test

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
estimates store fmodel
ologit warm i.yr89 c.ed c.prst, nolog
estimates store nmodel
lrtest fmodel nmodel

//  #7 scalar measures of fit using -fitstat-

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
rfitstat

//  #8 converting to a different parameterization (pg 164 RevEd)

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog

* intercept
lincom 0 - _b[/cut1]
* cutpoint 2
lincom _b[/cut2] - _b[/cut1]
* cutpoint 3
lincom _b[/cut3] - _b[/cut1]

//  #9 the parallel regression assumption

* generate data to demonstrate the assumption
clear
set obs 41
gen n = 41-_n
replace n = (n-20)/2.5
gen p1 = exp((-1*n)-3.5)/(1+exp((-1*n)-3.5))
gen p2 = exp((-1*n))/(1+exp((-1*n)))
gen p3 = exp((-1*n)+2)/(1+exp((-1*n)+2))
gen x = (n+8)*(9.375)
label var p1 "Pr(y<=1 | x)"
label var p2 "Pr(y<=2 | x)"
label var p3 "Pr(y<=3 | x)"
graph twoway (line p1 p2 p3 x) , ylabel(0(.2)1) xlabel(0(50)150) ///
    ytitle("Pr(y<=m)") xtitle("x")
graph export `exstub'05parallel.emf, replace

* approximate LR test
* note: omodel is an add-on Stata command. Type: findit omodel
sysuse ordwarm4, clear
*FV not allowed: omodel logit warm i.yr89 i.male i.white c.age c.ed c.prst

* Wald test
ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
est store olm1
rbrant, detail

//  gologit for LR and wald test

*fv not allowed
gologit2 warm yr89 male white age ed prst, npl store(golm1)
* wald test of equality of coefficients
test [#1=#2=#3]
foreach nm in yr89 male white age ed prst {
    test [#1=#2=#3] : `nm'
}
* lrtest between ologit & gologit2, npl
lrtest olm1 golm1, stats force

//  #10 residuals and outliers with -predict-

sysuse ordwarm4, clear
drop warmlt2 warmlt3 warmlt4
gen warmlt2 = (warm<2) if warm <.
gen warmlt3 = (warm<3) if warm <.
gen warmlt4 = (warm<4) if warm <.

* binary logit of warm < 2
logit warmlt2 i.yr89 i.male i.white c.age c.ed c.prst, nolog
predict rstd_lt2, rs

* binary logit of warm < 3
logit warmlt3 i.yr89 i.male i.white c.age c.ed c.prst, nolog
predict rstd_lt3, rs

* binary logit of warm < 4
logit warmlt4 i.yr89 i.male i.white c.age c.ed c.prst, nolog
predict rstd_lt4, rs

* binary logit of plot results for warm < 3
sort prst
gen index = _n
graph twoway scatter rstd_lt3 index, yline(0) ylabel(-4(2)4) ///
    xtitle("Observation Number")  xlabel(0(500)2293) msymbol(Oh)
graph export `exstub'05residplot.emf, replace

//  #11 partial change in y* (pg  Re RevEd)

rlistcoef, std help

//  #12 in sample predicted probabilities with -predict-

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog

*!? why does ologit have to be run just before this?
predict SDwarm Dwarm Awarm SAwarm
label var SDwarm "Pr(SD)"
label var Dwarm "Pr(D)"
label var Awarm "Pr(A)"
label var SAwarm "Pr(SA)"
dotplot SDwarm Dwarm Awarm SAwarm, ylabel(0(.25).75)
graph export `exstub'05pred.emf, replace

//  #13 individual predicted probabilities with prvalue

* working class men in 1977 who are near retirement
*fv error: margins, at(i.yr89=0 male=1 prst=20 age=64 ed=16) atmeans
*fv error: margins, at(1.yr89=0 1.male=1 prst=20 age=64 ed=16) atmeans

mtable , at(yr89=0 male=1 prst=20 age=64 ed=16) atmeans mlistat

*fv: if 1. you must specify 0. as well
margins, at(1.yr89=0 0.yr89=1) atmeans

* young, highly educated women with prestigious jobs
foreach w in 1 2 3 4 {
    margins, predict(outcome(`w')) ///
        at(yr89=1 male=0 prst=80 age=30 ed=24) atmeans
}
* or
mtable , at(yr89=1 male=0 prst=80 age=30 ed=24) mlistat atmeans

* an average individual in 1977 and 1989
mtable , at(yr89=0) at(yr89=1)  atmeans mlistat

//  #14 tables of predicted probabilities

* confidence intervals
mtable, at(yr89=(0 1) male=(0 1)) atmeans atvar(1.yr89 1.male)

//  #15 confidence intervals for discrete change

* men compared to women
estimates store olm1
foreach v in 1 2 3 4 {
    margins, atmeans at(yr89=1 male=(0 1)) post predict(outcome(`v'))
    mlincom, exp(2-1)
    estimates restore olm1
}
* or the spost way...
mtable, atmeans dydx(male) at(yr89=1)

* standard deviation change in age
di 44.935 - (.5*16.779)
di 44.935 + (.5*16.779)
foreach v in 1 2 3 4 {
    margins, atmeans at(male=0 yr89=1 age=36.5455) ///
        at(male=0 yr89=1 age=53.3245) post predict(outcome(`v'))
    mlincom, exp(2-1)
    estimates restore olm1
}
mtable, atmeans at(male=0 yr89=1 age=36.5455) ///
    at(male=0 yr89=1 age=53.3245) noci
* use mata to compute difference:

//  #16 graphing predicted probabilities with -prgen- (pg 177 RevEd)

mgen, at(age=(20(5)80) male=0 yr89=1) atmeans stub(w89)
desc w89*
label var w89p1 "SD"
label var w89p2 "D"
label var w89p3 "A"
label var w89p4 "SA"
label var w89Cp1 "SD"
label var w89Cp2 "SD or D"
label var w89Cp3 "SD, D or A"

* step 1: graph predicted probabilities
graph twoway connected w89p1 w89p2 w89p3 w89p4 w89age, ///
     title("Panel A: Predicted Probabilities") ///
     xtitle("Age") xlabel(20(10)80) ylabel(0(.25).50) ///
     yscale(noline) ylabel("") xline(44.93) ///
     ytitle("") name(tmp1, replace)

* step 2: graph cumulative probabilities
graph twoway connected w89Cp1 w89Cp2 w89Cp3 w89age, ///
    title("Panel B: Cumulative Probabilities") ///
    xtitle("Age") xlabel(20(10)80) ylabel(0(.25)1) xline(44.93) ///
    yscale(noline) ylabel("") name(tmp2, replace) ///
    ytitle("")

* step 3: combine graphs
graph combine tmp1 tmp2, col(1) iscale(*.9) imargin(small) ///
    ysize(4.31) xsize(3.287)

* step 4: save graph as eps
graph export `exstub'05prgen.emf, replace

//  #17 changes in predicted probabilities with prchange

sysuse ordwarm4, clear
ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog

* marginal change
*fv: does not require 1.male 1.yr89
mchange age, at(male=0 yr89=1) atmeans

* discrete change
mchange male age prst, at(male=0 yr89=1) atmeans

* using -delta- to compute effect of 10-year increase in age
mchange age, at(male=0 yr89=1) atmeans delta(10)

//  #18 odds ratios using listcoef

ologit warm i.yr89 i.male i.white c.age c.ed c.prst, nolog
*fv error: rlistcoef male age, help
rlistcoef i.male age, help
rlistcoef 1.male age, help
rlistcoef i.male age, help percent
rlistcoef i.male, reverse

log close
exit